#include "vtkTestProperty.h"
 
#include "vtkCommand.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"
#include "vtkStreamingDemandDrivenPipeline.h"
 
#include "vtkPolyData.h"

//vtkCxxRevisionMacro(vtkTestProperty, "$Revision: 1.1 $");
vtkStandardNewMacro(vtkTestProperty);
 
//----------------------------------------------------------------------------
vtkTestProperty::vtkTestProperty()
{
  this->SetNumberOfInputPorts( 1 );
  this->SetNumberOfOutputPorts( 1 );

  this->NPoints=-999;
  vtkPolyData *output = vtkPolyData::New();
  this->SetOutput(output);
  // Releasing data for pipeline parallism.
  // Filters will know it is empty.
  output->ReleaseData();
  output->Delete();
}
 
//----------------------------------------------------------------------------
vtkTestProperty::~vtkTestProperty()
{
}
 
//----------------------------------------------------------------------------
void vtkTestProperty::PrintSelf(ostream& os, vtkIndent indent)
{
  this->Superclass::PrintSelf(os, indent);
}
 
//----------------------------------------------------------------------------
vtkPolyData* vtkTestProperty::GetOutput()
{
cerr<<"vtkTestProperty::GetOutput() "<<endl;
  return this->GetOutput(0);
}
 
//----------------------------------------------------------------------------
vtkPolyData* vtkTestProperty::GetOutput(int port)
{
cerr<<"vtkTestProperty::GetOutput("<<port<<") "<<endl;
  return vtkPolyData::SafeDownCast(this->GetOutputDataObject(port));
}
 
//----------------------------------------------------------------------------
void vtkTestProperty::SetOutput(vtkDataObject* d)
{
cerr<<"vtkTestProperty::SetOutput("<<(long)d<<")"<<endl;
  this->GetExecutive()->SetOutputData(0, d);
}
 
 
//----------------------------------------------------------------------------
int vtkTestProperty::ProcessRequest(vtkInformation* request,
                                     vtkInformationVector** inputVector,
                                     vtkInformationVector* outputVector)
{
  cerr<<"vtkTestProperty::ProcessRequest "<<(*request)<<endl;
  // Create an output object of the correct type.
  if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA_OBJECT()))
  {
    return this->RequestDataObject(request, inputVector, outputVector);
  }
  // generate the data
  if(request->Has(vtkDemandDrivenPipeline::REQUEST_DATA()))
  {
    return this->RequestData(request, inputVector, outputVector);
  }
 
  if(request->Has(vtkStreamingDemandDrivenPipeline::REQUEST_UPDATE_EXTENT()))
  {
    return this->RequestUpdateExtent(request, inputVector, outputVector);
  }
 
  // execute information
  if(request->Has(vtkDemandDrivenPipeline::REQUEST_INFORMATION()))
  {
    return this->RequestInformation(request, inputVector, outputVector);
  }
 
  return this->Superclass::ProcessRequest(request, inputVector, outputVector);
}
 
//----------------------------------------------------------------------------
int vtkTestProperty::FillOutputPortInformation(
    int port, vtkInformation* info)
{
  cerr<<"vtkTestProperty::FillOutputPortInformation "<<port<<" "<<*info<<endl;
  // now add our info
  info->Set(vtkDataObject::DATA_TYPE_NAME(), "vtkPolyData");
  return 1;
}

int vtkTestProperty::FillInputPortInformation(
    int port, vtkInformation* info)
{
  cerr<<"vtkTestProperty::FillInputPortInformation "<<port<<" "<<*info<<endl;
  // now add our info
  info->Set(vtkAlgorithm::INPUT_REQUIRED_DATA_TYPE(), "vtkPolyData");
  return 1;
}
 

//----------------------------------------------------------------------------
int vtkTestProperty::RequestDataObject(
                                        vtkInformation* request,
    vtkInformationVector** vtkNotUsed(inputVector),
                                      vtkInformationVector* outputVector )
{
  cerr<<"vtkTestProperty::RequestDataObject "<<(*request)<<endl;
  for ( int i = 0; i < this->GetNumberOfOutputPorts(); ++i )
  {
    vtkInformation* outInfo = outputVector->GetInformationObject( i );
    vtkPolyData* output = vtkPolyData::SafeDownCast(
                                            outInfo->Get( vtkDataObject::DATA_OBJECT() ) );
    if ( ! output )
    {
      output = vtkPolyData::New();
      outInfo->Set( vtkDataObject::DATA_OBJECT(), output );
      output->FastDelete();
      //output->SetPipelineInformation( outInfo );
      this->GetOutputPortInformation( i )->Set(
                                      vtkDataObject::DATA_EXTENT_TYPE(), output->GetExtentType() );
    }
  }
  return 1;
}


//----------------------------------------------------------------------------
int vtkTestProperty::RequestInformation(
                                         vtkInformation* request,
    vtkInformationVector** inputVector,
                                      vtkInformationVector* vtkNotUsed(outputVector))
{
  cerr<<"vtkTestProperty::RequestInformation "<<(*request)<<endl;
    vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
    vtkPolyData *input = vtkPolyData::SafeDownCast(
                                         inInfo->Get(vtkDataObject::DATA_OBJECT()));

cerr<<"input verts: "<<input->GetNumberOfPoints()<<endl;
cerr<<"input lines: "<<input->GetNumberOfLines()<<endl;
cerr<<"input polys: "<<input->GetNumberOfPolys()<<endl;
cerr<<"input cells: "<<input->GetNumberOfCells()<<endl;
cerr<<"input points: "<<input->GetNumberOfPoints()<<endl;
    int nold=this->GetNPoints();
    int nset=input->GetNumberOfPoints();
    this->SetNPoints(nset);
    int nnew=this->GetNPoints();
    cerr<<"Setting NPoints:="<<nset<<" (before: "<<nold<<", after: "<<nnew<<")"<<endl;
  return 1;
}
 
//----------------------------------------------------------------------------
int vtkTestProperty::RequestUpdateExtent(
                                          vtkInformation* request,
    vtkInformationVector** inputVector,
    vtkInformationVector* vtkNotUsed(outputVector))
{
  cerr<<"vtkTestProperty::RequestUpdateExtent "<<(*request)<<endl;
  int numInputPorts = this->GetNumberOfInputPorts();
  for (int i=0; i<numInputPorts; i++)
  {
    int numInputConnections = this->GetNumberOfInputConnections(i);
    for (int j=0; j<numInputConnections; j++)
    {
      vtkInformation* inputInfo = inputVector[i]->GetInformationObject(j);
      inputInfo->Set(vtkStreamingDemandDrivenPipeline::EXACT_EXTENT(), 1);
    }
  }
  return 1;
}
 
//----------------------------------------------------------------------------
// This is the superclasses style of Execute method.  Convert it into
// an imaging style Execute method.
int vtkTestProperty::RequestData(
                                  vtkInformation* request,
    vtkInformationVector** inputVector,
                                       vtkInformationVector* outputVector )
{
  cerr<<"vtkTestProperty::RequestData "<<(*request)<<endl;
  for ( int i = 0; i < this->GetNumberOfOutputPorts(); ++i )
  {
    vtkInformation* outInfo = outputVector->GetInformationObject( i );
    vtkPolyData* output = vtkPolyData::SafeDownCast(
                                            outInfo->Get( vtkDataObject::DATA_OBJECT() ) );
    if ( ! output )
    {
      cout<<"vtkTestProperty::RequestData CREATING OUTPUT"<<endl;
      output = vtkPolyData::New();
      outInfo->Set( vtkDataObject::DATA_OBJECT(), output );
      output->FastDelete();
      //output->SetPipelineInformation( outInfo );
      this->GetOutputPortInformation( i )->Set(
                                      vtkDataObject::DATA_EXTENT_TYPE(), output->GetExtentType() );
    } else {
      cout<<"vtkTestProperty::RequestData REUSING OUTPUT"<<endl;
    }

    vtkInformation *inInfo = inputVector[0]->GetInformationObject(0);
    vtkPolyData *input = vtkPolyData::SafeDownCast(
                                         inInfo->Get(vtkDataObject::DATA_OBJECT()));

cerr<<"input verts: "<<input->GetNumberOfPoints()<<endl;
cerr<<"input lines: "<<input->GetNumberOfLines()<<endl;
cerr<<"input polys: "<<input->GetNumberOfPolys()<<endl;
cerr<<"input cells: "<<input->GetNumberOfCells()<<endl;
cerr<<"input points: "<<input->GetNumberOfPoints()<<endl;
    output->ShallowCopy(input);
    int nold=this->GetNPoints();
    int nset=input->GetNumberOfPoints();
    //this->SetNPoints(nset);
    int nnew=this->GetNPoints();
    cerr<<"(NOT) Setting NPoints:="<<nset<<" (before: "<<nold<<", after: "<<nnew<<")"<<endl;
  }

  // do nothing let subclasses handle it
  return 1;
}
